using System;
using System.Text;

namespace RepPP.Toolkit {
	/// <summary>
	/// TypedInstanceGenForCS:  Typed instance generator for C#.
	/// </summary>
	public class TypedInstanceGenForCS : TypedInstanceGenBase {

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public TypedInstanceGenForCS() : base(new Web.WebServicesGenForCS()) {
		}

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedGroupInstance:      Generate typed instance for a group
        /// </summary>
        /// <param name="cgrp">                 Card group</param>
        /// <param name="grp">                  Group</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="iLevel">               Right pad level</param>
        /// <param name="strBaseClassName">     Name of the base class</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************        
		private string IntGenerateTypedGroupInstance(RepPP.CardGroup   cgrp,
		                                             RepPP.Group       grp,
                                                     string            strCrdClassPrefix,
                                                     string            strGrpClassPrefix,
                                                     string            strGrpPropertyPrefix,
                                                     string            strFldPropertyPrefix,
                                                     string            strNameSpace,
                                                     string            strBaseClassName,
		                                             int               iLevel,
		                                             bool              bGenWS) {
            StringBuilder           strBuilder;
            Web.WebServicesGenBase  wsGen;
            string                  strFldPropName;
            string                  strGrpName;
            string                  strGrpPropName;
            string                  strClassName;
            string                  strChildClassName;
            string                  strContainerClassName;
            string                  strGroupBaseClassName;
            string                  strIndent;
            string                  strFldType;
            int                     iCount;
            int                     iIndex;
            string                  strUserClassName;
            
            wsGen                 = WSGenerator;
            strIndent             = "".PadRight(iLevel * 4, ' ');
            strBuilder            = new StringBuilder(null, 4096);            
            iCount                = grp.Fields.Count;
            strClassName          = DecorateGroupClassName(TYPED_GROUP_INST_PREFIX + strGrpClassPrefix, (cgrp != null) ? GetCardGroupName(cgrp) : GetGroupName(grp));
            strGroupBaseClassName = (strBaseClassName != String.Empty) ? strBaseClassName : "RepPP.TypedGroupInstanceBase";
            strUserClassName      = DecorateGroupClassName(strGrpClassPrefix, (cgrp != null) ? GetCardGroupName(cgrp) : GetGroupName(grp));
            strBuilder.Append(strIndent + "/// <summary>\r\n");
            strBuilder.Append(strIndent + "/// Typed Instance of group " + grp.Name + "\r\n");
            strBuilder.Append(strIndent + "/// </summary>\r\n");
            if (cgrp == null) {
                strBuilder.Append(strIndent + "[RepPP.TypedInstanceClass(\"" + grp.Name + "\", 100)]\r\n");
            }
            strBuilder.Append(strIndent + "public class " + strClassName + " : " + strGroupBaseClassName + " {\r\n");
            strBuilder.Append(strIndent + "    /// <summary>\r\n");
            strBuilder.Append(strIndent + "    /// Class constructor.\r\n");
            strBuilder.Append(strIndent + "    /// </summary>\r\n");
            strBuilder.Append(strIndent + "    /// <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append(strIndent + "    public " + strClassName + "(Int32 pobj) : \r\n");
            if (cgrp == null) {
                strBuilder.Append(strIndent + "                             base(pobj,\r\n");
                strBuilder.Append(strIndent + "                                  new string[] {\r\n");
                iIndex = 0;
                foreach (RepPP.Field fld in grp.Fields) {
                    iIndex++;
                    strBuilder.Append(strIndent + "                                                \"" + fld.Name + "\"" + 
                                      ((iIndex != iCount) ? "," : "") + "\r\n");
                }
                strBuilder.Append(strIndent + "                                               }) {\r\n");
            } else {
                strBuilder.Append(strIndent + "                             base(pobj) {\r\n");
            }
            strBuilder.Append(strIndent + "    }\r\n");
            if (cgrp == null) {            
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    /// <summary>\r\n");
                strBuilder.Append(strIndent + "    /// Create:               Create a new instance.\r\n");
                strBuilder.Append(strIndent + "    /// </summary>\r\n");
                strBuilder.Append(strIndent + "    /// <param name=\"app\">    Rep++ application object</param>\r\n");
                strBuilder.Append(strIndent + "    /// <returns>\r\n");
                strBuilder.Append(strIndent + "    /// New instance\r\n");
                strBuilder.Append(strIndent + "    /// </returns>\r\n");
                strBuilder.Append(strIndent + "    public static " + strUserClassName + " Create(RepPP.Application app) {\r\n");
                strBuilder.Append(strIndent + "        return(app.Groups[\"" + grp.Name + "\"].GroupInstances.Create(RepPP.GroupType.sdHorizontal) as " + strUserClassName + ");\r\n");
                strBuilder.Append(strIndent + "    }\r\n");

                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    /// <summary>\r\n");
                strBuilder.Append(strIndent + "    /// Create:               Create a new instance of the specified type.\r\n");
                strBuilder.Append(strIndent + "    /// </summary>\r\n");
                strBuilder.Append(strIndent + "    /// <param name=\"app\">    Rep++ application object</param>\r\n");
                strBuilder.Append(strIndent + "    /// <param name=\"type\">   Type of the wrapper class to create</param>\r\n");
                strBuilder.Append(strIndent + "    /// <returns>\r\n");
                strBuilder.Append(strIndent + "    /// New instance\r\n");
                strBuilder.Append(strIndent + "    /// </returns>\r\n");
                strBuilder.Append(strIndent + "    protected static " + strUserClassName + " Create(RepPP.Application app, Type type) {\r\n");
                strBuilder.Append(strIndent + "        RepPP.GroupInstance         giRetVal;\r\n");
                strBuilder.Append(strIndent + "        RepPP.Group                 grp;\r\n");
                strBuilder.Append(strIndent + "        Type                        typeOld;\r\n");
                strBuilder.Append(strIndent + "        RepPP.TypedInstanceFactory  typedInstFact;\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "        grp             = app.Groups[\"" + grp.Name + "\"];\r\n");
                strBuilder.Append(strIndent + "        typedInstFact   = app.GetObjFactory(typeof(RepPP.GroupInstance)) as RepPP.TypedInstanceFactory;\r\n");
                strBuilder.Append(strIndent + "        typeOld         = typedInstFact.GetTypeAssociateWithGroup(grp);\r\n");
                strBuilder.Append(strIndent + "        typedInstFact.AddGroup(grp, type);\r\n");
                strBuilder.Append(strIndent + "        try {\r\n");
                strBuilder.Append(strIndent + "            giRetVal = grp.GroupInstances.Create(RepPP.GroupType.sdHorizontal);\r\n");
                strBuilder.Append(strIndent + "        } finally {\r\n");
                strBuilder.Append(strIndent + "            typedInstFact.AddGroup(grp, typeOld);\r\n");
                strBuilder.Append(strIndent + "        }\r\n");
                strBuilder.Append(strIndent + "        return(giRetVal as " + strUserClassName + ");\r\n");
                strBuilder.Append(strIndent + "    }\r\n");
                foreach (RepPP.Field fld in grp.Fields) {
                    strFldType      = fld.GetType().Name.Replace("Field", "FieldInstance");
                    strFldPropName  = DecorateFieldPropertyName(strFldPropertyPrefix, GetFieldName(fld));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    /// <summary>\r\n");
                    strBuilder.Append(strIndent + "    /// " + fld.Prompt + "\r\n");
                    strBuilder.Append(strIndent + "    /// </summary>\r\n");
                    strBuilder.Append(strIndent + "    public RepPP." + strFldType + " " + strFldPropName + " {\r\n");
                    strBuilder.Append(strIndent + "        get {\r\n");
                    strBuilder.Append(strIndent + "            return(FieldInstances[\"" + fld.Name + "\"] as RepPP." + strFldType + ");\r\n");
                    strBuilder.Append(strIndent + "        }\r\n");
                    strBuilder.Append(strIndent + "    }\r\n");
                }
                if (bGenWS) {
                    strContainerClassName = (cgrp == null) ? strClassName : DecorateCardClassName(strCrdClassPrefix, GetCardName(cgrp.Card));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateGroupSerializationFunctions(strContainerClassName, cgrp, grp, iLevel * 4 + 4));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateGroupClass(grp, iLevel * 4 + 4, "", strGrpClassPrefix));
                }
            } else {
                foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
                    strGrpName          = cgrpChild.Group.Name;
                    strGrpPropName      = DecorateGroupPropertyName(strGrpPropertyPrefix, GetCardGroupName(cgrpChild));
                    strChildClassName   = DecorateGroupClassName(strGrpClassPrefix, GetCardGroupName(cgrpChild));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    /// <summary>\r\n");
                    strBuilder.Append(strIndent + "    /// Gets the wrapper for the instance of group " + strGrpName + "\r\n");
                    strBuilder.Append(strIndent + "    /// </summary>\r\n");
                    strBuilder.Append(strIndent + "    public " + strChildClassName + " " + strGrpPropName + " {\r\n");
                    strBuilder.Append(strIndent + "        get {\r\n");
                    strBuilder.Append(strIndent + "            RepPP.GroupInstance  ginstChild;\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "            ginstChild = (CurrentLine != -1) ? ChildGroupInstances[\"" + strGrpName + "\"] : null;\r\n");
                    strBuilder.Append(strIndent + "            return(ginstChild as " + strChildClassName + ");\r\n");
                    strBuilder.Append(strIndent + "        }\r\n");
                    strBuilder.Append(strIndent + "    }\r\n");
                }
                foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
		            strBuilder.Append(IntGenerateTypedGroupInstance(cgrpChild,
		                                                            cgrpChild.Group,
		                                                            strCrdClassPrefix,
		                                                            strGrpClassPrefix,
		                                                            strGrpPropertyPrefix,
		                                                            strFldPropertyPrefix,
                                                                    strNameSpace,
                                                                    strNameSpace + "." + DecorateGroupClassName(strGrpClassPrefix, GetGroupName(cgrpChild.Group)),
		                                                            iLevel + 1,
		                                                            bGenWS));
                }
            }
            strBuilder.Append(strIndent + "} // Class " + strClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateGroupInstanceProperty:      Generate the group instances properties
        /// </summary>
        /// <param name="cgrp">                 Card group</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <returns>
        /// The properties code
        /// </returns>
        //
        //******************************************************************
        private string GenerateGroupInstanceProperty(RepPP.CardGroup    cgrp,
                                                     string             strGrpClassPrefix,
                                                     string             strGrpPropertyPrefix) {
            StringBuilder   strBuilder;
            string          strGroupClassName;
            string          strGrpName;
            string          strGrpPropName;

            strBuilder          = new StringBuilder(null, 4096);            
            strGrpName          = cgrp.Group.Name;
            strGrpPropName      = DecorateGroupPropertyName(strGrpPropertyPrefix, GetCardGroupName(cgrp));
            strGroupClassName   = DecorateGroupClassName(strGrpClassPrefix, GetGroupName(cgrp.Group));
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append("        /// Gets the wrapper for the instance of group " + strGrpName + "\r\n");
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append("        public " + strGroupClassName + " " + strGrpPropName + " {\r\n");
            strBuilder.Append("            get {\r\n");
            if (cgrp.Parent == null) {
                strBuilder.Append("                return(RootGroupInstance as " + strGroupClassName + ");\r\n");
            } else {
                strBuilder.Append("                return(GetCurrentGroupInstance(Card.FindCardGroup(\"" + strGrpName + "\")) as " + strGroupClassName + ");\r\n");
            }
            strBuilder.Append("            }\r\n");
            strBuilder.Append("        }\r\n");
            foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
                strBuilder.Append(GenerateGroupInstanceProperty(cgrpChild, strGrpClassPrefix, strGrpPropertyPrefix));
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedCardInstance:       Generate typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strCrdBaseClassName">  Name of the base class for the card</param>
        /// <param name="strGrpBaseClassName">  Name of the base class for the groups</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		private string IntGenerateTypedCardInstance(RepPP.Card     crd,
                                                    string         strCrdClassPrefix,
                                                    string         strGrpClassPrefix,
                                                    string         strGrpPropertyPrefix,
                                                    string         strFldPropertyPrefix,
                                                    string         strCrdBaseClassName,
                                                    string         strGrpBaseClassName,
                                                    bool           bGenWS) {
            StringBuilder           strBuilder;
            Web.WebServicesGenBase  wsGen;
            RepPP.CardGroup         cgrpRoot;
            string                  strCardClassName;
            string                  strBaseClassName;
            string                  strUserCardClassName;
            
            wsGen                = WSGenerator;
            strBuilder           = new StringBuilder(null, 4096);            
            strCardClassName     = DecorateCardClassName(TYPED_CARD_INST_PREFIX + strCrdClassPrefix, GetCardName(crd));
            strBaseClassName     = (strCrdBaseClassName != String.Empty) ? strCrdBaseClassName : "RepPP.TypedCardInstanceBase";
            strUserCardClassName = DecorateCardClassName(strCrdClassPrefix, GetCardName(crd));
            strBuilder.Append("    /// <summary>\r\n");
            strBuilder.Append("    /// Typed Instance of card " + crd.Name + "\r\n");
            strBuilder.Append("    /// </summary>\r\n");
            strBuilder.Append("    [RepPP.TypedInstanceClass(\"" + crd.Name + "\", 100)]\r\n");
            strBuilder.Append("    public class " + strCardClassName + " : " + strBaseClassName + " {\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append("        /// Class constructor.\r\n");
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append("        /// <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append("        public " + strCardClassName + "(Int32 pobj) : base(pobj) {\r\n");
            strBuilder.Append("        }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append("        /// Create:               Create a new instance.\r\n");
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append("        /// <param name=\"app\">    Rep++ application object</param>\r\n");
            strBuilder.Append("        /// <returns>\r\n");
            strBuilder.Append("        /// New instance\r\n");
            strBuilder.Append("        /// </returns>\r\n");
            strBuilder.Append("        public static " + strUserCardClassName + " Create(RepPP.Application app) {\r\n");
            strBuilder.Append("            return(app.Cards[\"" + crd.Name + "\"].CardInstances.Add() as " + strUserCardClassName + ");\r\n");
            strBuilder.Append("        }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        /// <summary>\r\n");
            strBuilder.Append("        /// Create:               Create a new instance of the specified type.\r\n");
            strBuilder.Append("        /// </summary>\r\n");
            strBuilder.Append("        /// <param name=\"app\">    Rep++ application object</param>\r\n");
            strBuilder.Append("        /// <param name=\"type\">   Type of the wrapper class to create</param>\r\n");
            strBuilder.Append("        /// <returns>\r\n");
            strBuilder.Append("        /// New instance\r\n");
            strBuilder.Append("        /// </returns>\r\n");
            strBuilder.Append("        protected static " + strUserCardClassName + " Create(RepPP.Application app, Type type) {\r\n");
            strBuilder.Append("            RepPP.CardInstance          ciRetVal;\r\n");
            strBuilder.Append("            RepPP.Card                  crd;\r\n");
            strBuilder.Append("            Type                        typeOld;\r\n");
            strBuilder.Append("            RepPP.TypedInstanceFactory  typedInstFact;\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("            crd             = app.Cards[\"" + crd.Name + "\"];\r\n");
            strBuilder.Append("            typedInstFact   = app.GetObjFactory(typeof(RepPP.CardInstance)) as RepPP.TypedInstanceFactory;\r\n");
            strBuilder.Append("            typeOld         = typedInstFact.GetTypeAssociateWithCard(crd);\r\n");
            strBuilder.Append("            typedInstFact.AddCard(crd, type);\r\n");
            strBuilder.Append("            try {\r\n");
            strBuilder.Append("                ciRetVal = crd.CardInstances.Add();\r\n");
            strBuilder.Append("            } finally {\r\n");
            strBuilder.Append("                typedInstFact.AddCard(crd, typeOld);\r\n");
            strBuilder.Append("            }\r\n");
            strBuilder.Append("            return(ciRetVal as " + strUserCardClassName + ");\r\n");
            strBuilder.Append("        }\r\n");
            cgrpRoot = crd.ChildCardGroups[0];
            if (cgrpRoot != null) {
                strBuilder.Append(GenerateGroupInstanceProperty(cgrpRoot, strGrpClassPrefix, strGrpPropertyPrefix));
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateCardSerializationFunctions(strCardClassName, crd, 8));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateCardClass(crd, 8, "", "", strGrpClassPrefix));
                }
            }
            strBuilder.Append("    } // class " + strCardClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedGroupInstance:     Generate user typed instance for a group
        /// </summary>
        /// <param name="grp">                  Group</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedGroupInstance(RepPP.Group   grp, 
                                                              string        strGrpClassPrefix,
                                                              string        strNameSpace,
                                                              string        strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce = true;
            string          strClassName;
            string          strBaseClassName;
            string          strIndent = "    ";

#if VC8
            bGenNamesapce = (strNameSpace != String.Empty);
#endif
            strBuilder = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("//"));
            strBuilder.Append("using System;\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("namespace " + strNameSpace + " {\r\n");
            }

            strClassName     = DecorateGroupClassName(strGrpClassPrefix, GetGroupName(grp));
            strBaseClassName = DecorateGroupClassName(TYPED_GROUP_INST_PREFIX + strGrpClassPrefix, GetGroupName(grp));
            strBuilder.Append(strIndent + "/// <summary>\r\n");
            strBuilder.Append(strIndent + "/// User typed Instance of group " + grp.Name + "\r\n");
            strBuilder.Append(strIndent + "/// </summary>\r\n");
            strBuilder.Append(strIndent + "[RepPP.TypedInstanceClass(\"" + grp.Name + "\", 200)]\r\n");
            strBuilder.Append(strIndent + "public class " + strClassName + " : " + strBaseClassName + " {\r\n");
            strBuilder.Append(strIndent + "    /// <summary>\r\n");
            strBuilder.Append(strIndent + "    /// Class constructor.\r\n");
            strBuilder.Append(strIndent + "    /// </summary>\r\n");
            strBuilder.Append(strIndent + "    /// <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append(strIndent + "    public " + strClassName + "(Int32 pobj) : base(pobj) {\r\n");
            strBuilder.Append(strIndent + "    }\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append(strIndent + "} // Class " + strClassName + "\r\n");

            if (bGenNamesapce) {
                strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedGroupInstance:         Generate typed instance for a group
        /// </summary>
        /// <param name="grp">                  Group</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <param name="strBaseClassName">     Name of the base class</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedGroupInstance(RepPP.Group   grp, 
                                                          string        strCrdClassPrefix,
                                                          string        strGrpClassPrefix,
                                                          string        strGrpPropertyPrefix,
                                                          string        strFldPropertyPrefix,
                                                          string        strNameSpace,
                                                          string        strRootNameSpace,
                                                          string        strBaseClassName,
                                                          bool          bGenWS) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce = true;

#if VC8
            bGenNamesapce = (strNameSpace != String.Empty);
#endif
            strBuilder = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("//"));
            strBuilder.Append("using System;\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("namespace " + strNameSpace + " {\r\n");
            }
            strBuilder.Append(IntGenerateTypedGroupInstance(null,
                                                            grp,
                                                            strCrdClassPrefix,
                                                            strGrpClassPrefix,
                                                            strGrpPropertyPrefix,
                                                            strFldPropertyPrefix,
                                                            strNameSpace,
                                                            strBaseClassName,
                                                            1,
                                                            bGenWS));
            if (bGenNamesapce) {
                strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedCardInstance:          Generate typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <param name="strCrdBaseClassName">  Name of the base class for the card instance</param>
        /// <param name="strGrpBaseClassName">  Name of the base class for the group instance</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedCardInstance(RepPP.Card     crd, 
                                                         string         strCrdClassPrefix,
                                                         string         strGrpClassPrefix,
                                                         string         strGrpPropertyPrefix,
                                                         string         strFldPropertyPrefix,
                                                         string         strNameSpace,
                                                         string         strRootNameSpace,
                                                         string         strCrdBaseClassName,
                                                         string         strGrpBaseClassName,
                                                         bool           bGenWS) {
            StringBuilder   strBuilder;
            RepPP.CardGroup cgrpRoot;
            bool            bGenNamespace = true;

#if VC8
            bGenNamespace = (strNameSpace != String.Empty);
#endif            
            strBuilder = new StringBuilder(null, 4096);            
            if (crd.ChildCardGroups.Count != 0) {
                cgrpRoot = crd.ChildCardGroups[0];
                strBuilder.Append(GenerateSignature("//"));
                strBuilder.Append("using System;\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamespace) {
                    strBuilder.Append("namespace " + strNameSpace + " {\r\n");
                }
                strBuilder.Append(IntGenerateTypedCardInstance(crd, 
                                                               strCrdClassPrefix,
                                                               strGrpClassPrefix,
                                                               strGrpPropertyPrefix,
                                                               strFldPropertyPrefix,
                                                               strCrdBaseClassName,
                                                               strGrpBaseClassName,
                                                               bGenWS));
                if (bGenNamespace) {
                    strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedCardInstance:     Generate user typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedCardInstance(RepPP.Card     crd, 
                                                             string         strCrdClassPrefix,
                                                             string         strNameSpace,
                                                             string         strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamespace = true;
            string          strCardClassName;
            string          strBaseClassName;

#if VC8
            bGenNamespace = (strNameSpace != String.Empty);
#endif            
            strBuilder = new StringBuilder(null, 4096);            
            if (crd.ChildCardGroups.Count != 0) {
                strBuilder.Append(GenerateSignature("//"));
                strBuilder.Append("using System;\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamespace) {
                    strBuilder.Append("namespace " + strNameSpace + " {\r\n");
                }

                strCardClassName  = DecorateCardClassName(strCrdClassPrefix, GetCardName(crd));
                strBaseClassName  = DecorateCardClassName(TYPED_CARD_INST_PREFIX + strCrdClassPrefix, GetCardName(crd));
                strBuilder.Append("    /// <summary>\r\n");
                strBuilder.Append("    /// User typed Instance of card " + crd.Name + "\r\n");
                strBuilder.Append("    /// </summary>\r\n");
                strBuilder.Append("    [RepPP.TypedInstanceClass(\"" + crd.Name + "\", 200)]\r\n");
                strBuilder.Append("    public class " + strCardClassName + " : " + strBaseClassName + " {\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("        /// <summary>\r\n");
                strBuilder.Append("        /// Class constructor.\r\n");
                strBuilder.Append("        /// </summary>\r\n");
                strBuilder.Append("        /// <param name=\"pobj\">    Reference to the internal object</param>\r\n");
                strBuilder.Append("        public " + strCardClassName + "(Int32 pobj) : base(pobj) {\r\n");
                strBuilder.Append("        }\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("    } // class " + strCardClassName + "\r\n");
                if (bGenNamespace) {
                    strBuilder.Append("} // namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all cards and groups which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		public override void RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, bool bFullSync) {
		    RegenerateTypedInstance(app, log, projFolder, ".cs", bFullSync);
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all cards and groups which need to be regenerated
		/// </summary>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		public override void RegenerateTypedInstance(ILog log, IProjectFolder projFolder, bool bFullSync) {
		    RegenerateTypedInstance(null, log, projFolder, ".cs", bFullSync);
		}
	} // class TypedInstanceGenForCS
} // namespace
