using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;
using RepPP.Toolkit;
using System.Xml;

namespace RepPP.Toolkit.Window {
    //*********************************************************
    //
	/// <summary>
	/// DataGridViewGInst:  Defines a DataGridView which is attached to a group instance
	/// </summary>
	//
    //*********************************************************
    public class DataGridViewGInst : DataGridView, CrdGrpControlUtil.ICrdGrpControl, ILineListHandler, IUIConfiguration {
        private DataSourceGInst                     m_dataSourceGInst;
        private LineSelectedInListEventArgs         m_evtArgLastRowChange = null;
        private FieldSelector                       m_fldSelector;
        /// <summary> Called when a line is selected in the LineListHandler</summary>
        public event LineSelectedInListEventHandler LineSelectedInList;


        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Build the class from a group. The source will be empty
        /// </summary>
        //
        //********************************************************************************
        public DataGridViewGInst() : base() {
            m_fldSelector       = new FieldSelector(this);
            m_dataSourceGInst   = new DataSourceGInst(this, false);
            base.DataSource     = m_dataSourceGInst;
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                    Releases the resources
        /// </summary>
        //
        //********************************************************************************
        protected override void Dispose(bool disposing)
        {
            if (disposing){
                if (m_dataSourceGInst != null) {
                    m_dataSourceGInst.Dispose();
                    m_dataSourceGInst = null;
                }
                if (m_fldSelector != null) {
                    m_fldSelector.Dispose();
                    m_fldSelector = null;
                }
            }
            base.Dispose(disposing);
        }
        //********************************************************************************
        //
        /// <summary>
        /// DataSourceGInst:            Returns the data source attached to a group instance
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("Data Source attached to a group instance")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        protected virtual DataSourceGInst DataSourceGInst {
            get {
                return(m_dataSourceGInst);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        RepPP.Application CrdGrpControlUtil.ICrdGrpControl.Application {
            get {
                return(DataSourceGInst.Application);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CardGroupFullName:                  Name of the CardGroup attached to the source
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor(typeof(CrdGrpControlUtil.CrdGrpControlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Name of the CardGroup attached to the source")]
        public CrdGrpFullName CardGroupFullName {
            get {
                return(DataSourceGInst.CardGroupFullName);
            }
            set {
                DataSourceGInst.CardGroupFullName = value;
                if (DesignMode) {
                    base.DataSource = null;
                    base.DataSource = m_dataSourceGInst;
                }
                SetColumnsInfo();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        void CrdGrpControlUtil.ICrdGrpControl.TriggerComponentChange() {
            DataSourceGInst.TriggerComponentChange();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized
		/// </summary>
		//
        //********************************************************************************
        void ControlUtil.IRepControl.RepPPReInit() {
            DataSourceGInst.RepPPReInit();
            SetColumnsInfo();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is resetted
		/// </summary>
		//
        //********************************************************************************
        void ControlUtil.IRepControl.RepPPReset() {
            DataSourceGInst.RepPPReset();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Group:              Attached group
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("Group which will be represented by this Data Source")]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual Group Group {
            get {
                return(DataSourceGInst.Group);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GroupInstance:      Attached group instance
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[Description("GroupInstance represented by this Data Source")]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public virtual GroupInstance GroupInstance {
            get {
                return(DataSourceGInst.GroupInstance);
            }
            set {
                DataSourceGInst.GroupInstance = value;
                base.DataSource               = DataSourceGInst;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:        Show description of choice if true, code if false
        /// </summary>
        //
        //********************************************************************************
		[Browsable(true)]
		[Category("Behavior")]
		[Description("How to display a choice value")]
        public bool ShowDescription {
            get {
                return(DataSourceGInst.ShowDescription == DataSourceGInst.ShowDescriptionE.Always);
            }
            set {
                DataSourceGInst.ShowDescription = (value) ? DataSourceGInst.ShowDescriptionE.Always : DataSourceGInst.ShowDescriptionE.Never;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetColumnInfo:                  Set the information of a column attached to a field
        /// </summary>
        /// <param name="col">              Column for which the information is set</param>
        /// <param name="fld">              Field attached to the column</param>
        /// <param name="bShowDescription"> true if show description instead of code</param>
        //
        //********************************************************************************
        protected virtual void SetColumnInfo(DataGridViewColumn col, RepPP.Field fld, bool bShowDescription) {
            RepPP.ChoiceList            choiceList;
            string                      strChoice;
            DataGridViewComboBoxColumn  cb;
            
            if (col is DataGridViewComboBoxColumn) {
                cb = (DataGridViewComboBoxColumn)col;
                choiceList = fld.ChoiceList;
                cb.Items.Clear();
                for (int iIndex = 0; iIndex < choiceList.Count; iIndex++) {
                    if (bShowDescription) {
                        strChoice = choiceList.GetDesc(iIndex);
                        if (strChoice == null || strChoice == String.Empty) {
                            strChoice = choiceList.GetCode(iIndex);
                        }
                    } else {
                        strChoice = choiceList.GetCode(iIndex);
                    }
                    cb.Items.Add(strChoice);
                }
            } else if (fld.Type == RepPP.FieldType.sdFieldMoney) {
                col.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            }
            if (col is DataGridViewTextBoxColumn) {
                ((DataGridViewTextBoxColumn)col).MaxInputLength = fld.MaxSize;
            }
            col.ReadOnly = !fld.IsSelectable || fld.IsPrimaryKey;
            col.Visible  = fld.IsVisible; 
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetColumnsInfo:                 Set the information of columns attached to a field
        /// </summary>
        //
        //********************************************************************************
        protected virtual void SetColumnsInfo() {
            RepPP.Group                     grp;
            RepPP.Field                     fld;
            string                          strPropName;
            bool                            bShowDescription;
            
            bShowDescription    = ShowDescription;
            grp                 = Group;
            if (grp != null) {
                foreach (DataGridViewColumn col in Columns) {
                    strPropName = col.DataPropertyName;
                    fld = (strPropName == null) ? null : grp.Fields[col.DataPropertyName];
                    if (fld != null) {
                        SetColumnInfo(col, fld, bShowDescription);
                    }   
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCreateControl:        Called when the control is created
        /// </summary>
        //
        //********************************************************************************
        protected override void  OnCreateControl() {
 	        base.OnCreateControl();
 	        SetColumnsInfo();
        }

        //********************************************************************************
        //
        /// <summary>
        /// FindColumnIndex:                Finds the index of a column using the name of a field
        /// </summary>
        /// <param name="fld">              Field</param>
        /// <returns>
        /// Index of the column or -1 if not found.
        /// </returns>
        //
        //********************************************************************************
        private int FindColumnIndex(Field fld) {
            int     iRetVal = -1;
            
            foreach (DataGridViewColumn col in Columns) {
                if (col.DataPropertyName == fld.Name) {
                    iRetVal = col.Index;
                    break;
                }
            }
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetGInstLineFromRowIndex:       Finds the index of a line in the group instance
        ///                                 using the index of a row in the grid
        /// </summary>
        /// <param name="iRowIndex">        the index of the row</param>
        /// <returns>
        /// Index of the line or -1 if not found.
        /// </returns>
        //
        //********************************************************************************
        private int GetGInstLineFromRowIndex(int iRowIndex){
            GroupInstance       gi;
            DataSourceGInst     dataSourceGInst;
            DataRowGInst        rowGInst;
            int                 iRetVal = -1;

            gi              = GroupInstance;
            dataSourceGInst = DataSourceGInst;
            if (iRowIndex >= 0 && iRowIndex < dataSourceGInst.Count){
                rowGInst = (dataSourceGInst == null) ? null : dataSourceGInst[iRowIndex] as DataRowGInst;
                if (gi != null && rowGInst != null) {
                    iRetVal = gi.LineFromBookmark(rowGInst.Bookmark);
                }
            }           
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetRowState:                    Enables the ReadOnly state of the PK cells in the 
        ///                                 current Row if it represents a new line
        /// </summary>
        /// <param name="iLineIndex">       the index of the line in the GroupInstance</param>
        /// <param name="iRowIndex">        the index of the row</param>
        //
        //********************************************************************************
        private void SetRowState(int iLineIndex, int iRowIndex){
            DataSourceGInst dataSourceGInst;
            int             iColumnIndex;

            if (GroupInstance.ThisLineState(iLineIndex) == LineState.sdLineNew){
                dataSourceGInst = DataSourceGInst;
                foreach(Field fld in dataSourceGInst.Fields) {
                    if (fld.IsPrimaryKey) {
                        iColumnIndex = FindColumnIndex(fld);
                        if (iColumnIndex != -1) {
                            Rows[iRowIndex].Cells[iColumnIndex].ReadOnly = false;                        
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnRowValidating:                Called when validation of a row occurs
        /// </summary>
        /// <param name="e">                Event parameter</param>
        //
        //********************************************************************************
        protected override void OnRowValidating(DataGridViewCellCancelEventArgs e) {
            GroupInstance   gi;
            DataGridViewRow row;
            int             iLineIndex;
            string          strError;
            string          str = "";
            int             iColIndex;
            
            base.OnRowValidating(e);
            if (!e.Cancel) {
                gi         = GroupInstance;
                iLineIndex = GetGInstLineFromRowIndex(e.RowIndex);
                if (gi != null && iLineIndex != -1) {
                    if (gi.ThisLineState(iLineIndex) != LineState.sdLineDeleted) {
                        gi.Group.SaveActiveGroupInstance();
                        gi.SaveLinePos();
                        try {
                            gi.Select(false, false);
                            gi.SelectLine(iLineIndex, true, false);
                            row             = Rows[e.RowIndex];
                            row.ErrorText   = "";
                            foreach (DataGridViewCell cell in row.Cells) {
                                cell.ErrorText = "";
                            }
                            strError                   = "";
                            foreach (FieldInstance fi in gi.FieldInstances) {
                                if (fi.Field.Validate(ref str) != 0) {
                                    e.Cancel    = true;
                                    iColIndex   = FindColumnIndex(fi.Field);
                                    if (iColIndex != -1) {
                                        row.Cells[iColIndex].ErrorText = str;
                                    }
                                    if (strError != String.Empty) {
                                        strError += "\r\n";
                                    }
                                    strError += fi.Field.Prompt + ": " + str;
                                }
                            }
                            if (e.Cancel) {
                                row.ErrorText = strError;
                            } else {
                                if (gi.ValidActiveLine(true) != 0) {
                                    e.Cancel = true;
                                }
                            }
                        } finally {
                            gi.RestoreLinePos(true);
                            gi.Group.RestoreActiveGroupInstance(true);
                        }
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:                     Ignore the setting of the datasource
        /// </summary>
        //
        //********************************************************************************
		[Browsable(false)]
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public new object DataSource {
            get {
                return(DataSourceGInst);
            }
            set {
                // Do nothing
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellEnter:                        Called before a cell got focus
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnCellEnter(DataGridViewCellEventArgs e) {
            string      strPropName;
            RepPP.Group grp;
            RepPP.Field fld;

            grp = Group;
            if (m_fldSelector != null && grp != null) {
                strPropName = Columns[e.ColumnIndex].DataPropertyName;
                fld         = (strPropName == null) ? null : grp.Fields[strPropName];
                if (fld != null) {
                    m_fldSelector.Select(fld, true);
                }
            }
            base.OnCellEnter(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCellLeave:                        Called before a cell loose focus
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnCellLeave(DataGridViewCellEventArgs e) {
            string      strPropName;
            RepPP.Group grp;
            RepPP.Field fld;

            grp = Group;
            if (m_fldSelector != null && grp != null) {
                strPropName = Columns[e.ColumnIndex].DataPropertyName;
                fld         = (strPropName == null) ? null : grp.Fields[strPropName];
                if (fld != null) {
                    m_fldSelector.Select(fld, false);
                }
            }
            base.OnCellLeave(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnRowEnter:                         Called before the line selection changes
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnRowEnter(DataGridViewCellEventArgs e) {
            int iOldLine = -1;
            int iNewLine = -1;

            base.OnRowEnter(e);
            if (CurrentRow != null) {
                iOldLine = GetGInstLineFromRowIndex(CurrentRow.Index);
            }
            iNewLine = GetGInstLineFromRowIndex(e.RowIndex);
            m_evtArgLastRowChange = new LineSelectedInListEventArgs(iOldLine, iNewLine);
            SetRowState(iNewLine, e.RowIndex);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnSelectionChanged:                 Called after the line selection changes
        /// </summary>
        /// <param name="e">                    Event Argument</param>
        //
        //********************************************************************************
        protected override void OnSelectionChanged(EventArgs e) {
            base.OnSelectionChanged(e);
            if (m_evtArgLastRowChange != null) {
                OnLineSelectedInList(m_evtArgLastRowChange);
            }
            m_evtArgLastRowChange = null;
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectLine:                     Select a row
        /// </summary>
        /// <param name="iPos">            Row position</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool SelectLine(int iPos) {
            DataGridViewRow row;
            bool            bRetVal;
            
            if (GroupInstance == null) {
                bRetVal = false;
            } else {
                ClearSelection();
                row = Rows[iPos];
                if (row.Cells.Count > 0) {
                    CurrentCell = row.Cells[0]; 
                    row.Selected = true;
                }
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InsertLine:                     Insert a row
        /// </summary>
        /// <param name="bAtEnd">           true to insert at the end of the list</param>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool InsertLine(bool bAtEnd) {
            DataGridViewRow row;
            bool            bRetVal;
            
            if (GroupInstance == null) {
                bRetVal = false;
            } else {
                if (!AllowUserToAddRows) {
                    DataSourceGInst.AddNew();
                    row = Rows[Rows.Count - 1];
                    if (row.Cells.Count > 0) {
                        CurrentCell = row.Cells[0];
                        CurrentCell.Selected = true;
                    }
                }
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DuplicateLine:                  Duplicate a row
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool DuplicateLine() {
            bool            bRetVal;
            DataGridViewRow row;
            
            if (GroupInstance == null) {
                bRetVal = false;
            } else {
                if (CurrentRow == null) {
                    bRetVal = false;
                } else {
                    DataSourceGInst.AddDuplicate();
                    row = Rows[Rows.Count - 1];
                    if (row.Cells.Count > 0) {
                        CurrentCell = row.Cells[0];
                        CurrentCell.Selected = true;
                    }
                    bRetVal             = true;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// DeleteCurrentLine:              Delete the current row
        /// </summary>
        /// <returns>
        /// true if succeed, false if failed
        /// </returns>
        //
        //********************************************************************************
        public bool DeleteCurrentLine() {
            bool    bRetVal;
            
            if (GroupInstance == null) {
                bRetVal = false;
            } else {
                if (CurrentRow != null) {
                    Rows.Remove(CurrentRow);
                    bRetVal = true;
                } else {
                    bRetVal = false;
                }
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// CommitEdit:                     Commit the edit
        /// </summary>
        //
        //********************************************************************************
        public void CommitEdit() {
            CommitEdit(DataGridViewDataErrorContexts.Commit);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Refresh:                        Refresh the content of the grid
        /// </summary>
        //
        //********************************************************************************
        void ILineListHandler.Refresh() {
            GroupInstance   ginst;
            
            ginst = GroupInstance;
            GroupInstance = null;
            GroupInstance = ginst;
        }

        //********************************************************************************
        //
        /// <summary>
        /// AssociateGroupInstance:         Associate a group instance with the grid
        /// </summary>
        /// <param name="ginst">            Group instance to associate</param>
        //
        //********************************************************************************
        public void AssociateGroupInstance(GroupInstance ginst) {
            GroupInstance = ginst;
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnLineSelectedInList:           Triggers the LineSelectedInList event
        /// </summary>
        /// <param name="e">                Event Argument</param>
        //
        //********************************************************************************
        public void OnLineSelectedInList(LineSelectedInListEventArgs e) {
            if (LineSelectedInList != null) {
                LineSelectedInList(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// BeginInit:                      Begins the initialization
        /// </summary>
        /// <remarks>
        /// Used to avoid an exception in design mode 
        /// </remarks>
        //
        //********************************************************************************
        public void BeginInit() {     //ISupportInitialize
        }

        //********************************************************************************
        //
        /// <summary>
        /// EndInit:                      Ends the initialization
        /// </summary>
        /// <remarks>
        /// Used to avoid an exception in design mode 
        /// </remarks>
        //
        //********************************************************************************
        public void EndInit() {     //ISupportInitialize
        }

        //********************************************************************************
        //
        /// <summary>
        /// LoadState:                      Load the control state
        /// </summary>
        /// <param name="helper">           Configuration helper</param>
        /// <param name="node">             Node containing the information</param>
        //
        //********************************************************************************
        void IUIConfiguration.LoadState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            string              strVersion;
            XmlNode             nodeColStyle;
            string              strColWidth;
            string              strDisplayIndex;
            int                 iWidth;
            int                 iDisplayIndex;
            
            strVersion = helper.GetNodeAttribute(node, "VERSION");
            if (strVersion != null && strVersion == "1") {
                foreach (DataGridViewColumn col in Columns) {
                    nodeColStyle = helper.FindNode(node, "col" + col.Name);
                    strColWidth  = helper.GetNodeAttribute(nodeColStyle, "WIDTH");
                    if (strColWidth != null && Int32.TryParse(strColWidth, out iWidth)) {
                        col.Width = iWidth;
                    }
                    strDisplayIndex = helper.GetNodeAttribute(nodeColStyle, "DISPLAYINDEX");
                    if (strDisplayIndex != null && Int32.TryParse(strDisplayIndex, out iDisplayIndex)) {
                        col.DisplayIndex = iDisplayIndex;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SaveState:                      Save the control state
        /// </summary>
        /// <param name="helper">           Configuration helper</param>
        /// <param name="node">             Node where to save the information</param>
        //
        //********************************************************************************
        void IUIConfiguration.SaveState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
            XmlNode nodeColStyle;

            node.RemoveAll();
            helper.SetNodeAttribute(node, "VERSION", "1");
            
            foreach (DataGridViewColumn col in Columns) {
                nodeColStyle = helper.FindNode(node, "col" + col.Name);
                helper.SetNodeAttribute(nodeColStyle, "WIDTH", col.Width.ToString());
                helper.SetNodeAttribute(nodeColStyle, "DISPLAYINDEX", col.DisplayIndex.ToString());
            }
        }
    } // Class DataGridViewGInst
} // Namespace
