//*********************************************************
//
// FldComboBox:		    Overload the microsoft .net combobox to implement 
//                      rep++ combobox field control
//
// Author:		        Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// SCFldComboBox: Create a combobox around a rep++ field
	/// </summary>
	public class FldComboBox : System.Windows.Forms.ComboBox, FldControlUtil.IFldControl
	{
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        protected bool                              m_bFilling;
        /// <summary>The maximum code length in the choice list</summary>
        protected int                               m_iMaxCodeLength;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldComboBox() {
			InitializeComponent();
            m_bFilling               = false;
            m_iMaxCodeLength         = 0;
            m_fldControlUtil         = new WndFldControlUtil(this, false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Items.Clear();
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
                InitializeCtlv();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor(typeof(FldControlUtil.FldControlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GroupName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string GroupName {
            get {
                return(FieldFullName.GroupName);
            }
            set {
                FieldFullName.GroupName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Field:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		    set {
		        m_fldControlUtil.Field = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control using the field properties
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            Field   fld;
            bool    bDesignMode;

            fld         = m_fldControlUtil.Field;
            bDesignMode = this.DesignMode;
            if (bDesignMode == false && fld != null) {
                m_bFilling = true;
                this.Enabled   = fld.IsSelectable;
                this.Visible   = fld.IsVisible;
                this.MaxLength = fld.MaxSize;
                if (fld.IsChoiceAvailable == true) {
                    BuildListItems(false);
                    this.SelectedIndex = fld.ChoiceList.FindValue(fld.Value);
                } else {
                    this.Text = fld.Value;
                }
                m_bFilling = false;
            } else if (bDesignMode == true) {
                m_bFilling  = true;
                this.Text   = m_fldControlUtil.FieldFullName.GroupName + "." +
                              m_fldControlUtil.FieldFullName.FieldName;
                m_bFilling  = false;
            }
        }
        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the TextBox
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                return base.Text;
            }
            set {
                base.Text = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetMaxCodeWidth:        Get the max width of the list codes
        ///                         measured using the current font
        /// </summary>
        /// <param name="lst">      Choice list</param>
        /// <returns>
        /// Max code width
        /// </returns>
        //
        //*********************************************************
        protected virtual int GetMaxCodeWidth(ChoiceList lst) {
            int         iRetVal  = 0;
            int         iIndex   = 0;
            string      strCode  = null;
            string      strValue = null;
            string      strDesc  = null;
            int         iCodeWidth;
            Graphics    gr;

            gr = CreateGraphics();
            do {
                iIndex = lst.ExtractChoice(iIndex, ref strValue, ref strCode, ref strDesc);
                if (iIndex != -1) {
                    iCodeWidth  = (int)gr.MeasureString(strCode, this.Font).Width;
                    if (iCodeWidth > iRetVal) {
                        iRetVal = iCodeWidth;
                        m_iMaxCodeLength = strCode.Length;
                    }
                }
            } while (iIndex != -1);
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// BuildListItemsv:        Build the combobox list items
        /// </summary>
        /// <param name="bForce">   Force the building of the list items</param>
        //
        //*********************************************************
        protected virtual void BuildListItems(bool bForce) {
            int         iIndex;
            int         iMaxCodeWidth;
            int         iSpaceWidth;
            int         iCodeWidth;
            string      strCode  = null;
            string      strDesc  = null;
            string      strValue = null;
            string      strChoiceAns;
            Field       fld;
            Graphics    gr;
            ChoiceList  lst;

            if (bForce || this.Items.Count == 0) {
                fld = m_fldControlUtil.Field;
                this.Items.Clear();
                if (fld != null && fld.IsChoiceAvailable == true) {
                    lst             = fld.ChoiceList;
                    gr              = CreateGraphics();
                    iSpaceWidth     = (int)gr.MeasureString(" ", Font).Width;
                    iMaxCodeWidth   = GetMaxCodeWidth(lst) + iSpaceWidth * 4;
                    iIndex          = 0;
                    do {
                        iIndex = lst.ExtractChoice(iIndex, ref strValue, ref strCode, ref strDesc);
                        if (iIndex != -1) {
                            if (strDesc != String.Empty) {
                                iCodeWidth   = (int)gr.MeasureString(strCode, Font).Width; 
                                strChoiceAns = strCode.PadRight(strCode.Length + (iMaxCodeWidth - iCodeWidth)/iSpaceWidth);
                                strChoiceAns = strChoiceAns + strDesc;
                            } else {
                                strChoiceAns = strCode;
                            }
                            this.Items.Add(strChoiceAns);
                        }
                    } while (iIndex != -1);
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        protected virtual void SetFieldValuev() {
            string  strCode;
            string  strValue;
            Field   fld;

            fld = m_fldControlUtil.Field;            
            if (fld != null) {
                if (fld.IsChoiceAvailable) {
                    strCode = this.Text;
                    if (strCode.Length > m_iMaxCodeLength) {
                        strCode = strCode.Substring(0, m_iMaxCodeLength);
                    }
                    strValue = fld.ChoiceList.ConvertCodeToValue(strCode);
                } else {
                    strValue = this.Text;
                }
                if (fld.Value != strValue) {
                    m_bFilling  = true;
                    fld.Value   = strValue;
                    m_bFilling  = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            this.Items.Clear();
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            Field = null;
            this.Items.Clear();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            Field   fld;

            fld = m_fldControlUtil.Field;
            if (fld != null && fld == e.Field && !m_bFilling) {
                if (e.FldEvent == FldEvent.sdFldChoiceListChg) {
                    BuildListItems(true);
                } else {
                    InitializeCtlv();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSelectedIndexChanged:     Raise the selected index changed event
        /// </summary>
        /// <param name="e">            Event arguments </param>
        //
        //*********************************************************
        protected override void OnSelectedIndexChanged(System.EventArgs e) {
            if (!m_bFilling) {
                SetFieldValuev();
            }
            base.OnSelectedIndexChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnTextChanged:      Raise the text changed event
        /// </summary>
        /// <param name="e">    Event arguments </param>
        //
        //*********************************************************
        protected override void OnTextChanged(System.EventArgs e) {
            if (!m_bFilling) {
                SetFieldValuev();
            }
            base.OnTextChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// IsInputChar:              Determines if a character is recognized by the control
        /// </summary>
        /// <param name="CharCode">   The character to test </param>
        //
        //*********************************************************
        protected override bool IsInputChar(char CharCode) {
            bool    bRetVal;
            Field   fld;

            fld = m_fldControlUtil.Field;
            if (CharCode >= 32 && CharCode <= 255) {
                bRetVal = this.Enabled == false && fld != null && fld.ValidateChar(CharCode);
            } else {
                bRetVal = base.IsInputChar(CharCode);
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                InitializeCtlv();
                if (!DesignMode) {
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // SCFldComboBox
} // namespace
