using System;
using System.Collections;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //*********************************************************	    
    //
    /// <summary>
    /// Implement the base class of a tree node attached to a TreeViewCInst 
    /// </summary>
    // 
    //*********************************************************	    
    public class TreeNodeText : TreeNodeBase {
        /// <summary>Array of child group instances attached to this node.</summary>
        private ArrayList                               m_arrChildGroupInstances;
        /// <summary>Node primary key if user want to specify one</summary>
        private string                                  m_strNodePK = null;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                                   Class constructor
        /// </summary>
        /// <param name="treeFather">               Father treeview</param>
        /// <param name="strText">                  Text to associate to the node</param>
        /// <param name="iNodeType">                Type to associate to the node</param>
        /// <param name="iImageIndex">              Index of the image for the unselected node</param>
        /// <param name="iSelectedImageIndex">      Index of the image for the selected node</param>
        /// <param name="arrChildGroupInstances">   Array of child group instances</param>
        /// <param name="childNodeFactory">         Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public TreeNodeText(TreeViewCInst                            treeFather,
                            string                                   strText,
                            int                                      iNodeType,
                            int                                      iImageIndex,
                            int                                      iSelectedImageIndex,
                            RepPP.GroupInstance[]                    arrChildGroupInstances,
                            TreeNodeGInst.IGInstChildNodeFactory     childNodeFactory) : base(strText, iNodeType, iImageIndex, iSelectedImageIndex, childNodeFactory)
        {
            Text                        = strText;
            m_arrChildGroupInstances    = new ArrayList();
            if (arrChildGroupInstances != null) {
                foreach (RepPP.GroupInstance ginst in arrChildGroupInstances) {
                    m_arrChildGroupInstances.Add(ginst);
                }
            }
            Init(treeFather, (arrChildGroupInstances != null && arrChildGroupInstances.Length != 0));
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                                   Class constructor
        /// </summary>
        /// <param name="treeFather">               Father treeview</param>
        /// <param name="strText">                  Text of the node or null to get the group instance values</param>
        /// <param name="arrChildGroupInstances">   Array of child group instances</param>
        /// <param name="childNodeFactory">         Node factory used to create child. Can be NULL</param>
        //
        //*********************************************************	    
        public TreeNodeText(TreeViewCInst                            treeFather,
                            string                                   strText,
                            RepPP.GroupInstance[]                    arrChildGroupInstances,
                            TreeNodeGInst.IGInstChildNodeFactory     childNodeFactory) : this(treeFather, strText, 0, -1, -1, arrChildGroupInstances, childNodeFactory)
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                Free the node
        /// </summary>
        //
        //*********************************************************	    
        public override void Dispose()
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Clone:                      Clone the node and all its descendant
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override object Clone() {
            TreeNodeText    nodeRetVal;

            nodeRetVal                          = base.Clone() as TreeNodeText;
            nodeRetVal.m_strNodePK              = m_strNodePK;
            nodeRetVal.m_arrChildGroupInstances = new ArrayList();
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// NodePK:                 Current node primary key if any
        /// </summary>
        //
        //*********************************************************	    
        public virtual string NodePK
        {
            get {
                return(m_strNodePK);
            }
            set {
                m_strNodePK = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetNodeKey:                 Gets the key of the node
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override string GetNodeKey()
        {
            return((m_strNodePK == null) ? "BASE:" + Text : m_strNodePK);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ChildGroupInstances:    Array of child group instances
        /// </summary>
        //
        //*********************************************************	    
        public ArrayList ChildGroupInstances {
            get {
                return(m_arrChildGroupInstances);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindChildGroupInstance:     Find a child group instance if any
        /// </summary>
        /// <param name="grp">          Group</param>
        /// <returns>
        /// null or the specified group instance
        /// </returns>
        //
        //*********************************************************	    
        public override RepPP.GroupInstance FindChildGroupInstance(RepPP.Group grp)
        {
            RepPP.GroupInstance ginstRetVal = null;
            
            foreach (RepPP.GroupInstance ginst in m_arrChildGroupInstances) {
                if (ginst.Group == grp) {
                    ginstRetVal = ginst;
                }
            }
            return(ginstRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ReadChildren:           Read node's children
        /// </summary>
        //
        //*********************************************************	    
        public override void ReadChildren()
        {
            TreeViewCInst.BeforeChildReadEventArgs  beforeArg;
            TreeViewCInst.AfterChildReadEventArgs   afterArg;
            
            if (!IsChildrenRead) {
                IsChildrenRead  = true;
                // Remove the 'Dummy' node created by the base class so the '+' appear
                Nodes.Clear();
                beforeArg       = new TreeViewCInst.BeforeChildReadEventArgs(this);
                OnBeforeChildRead(beforeArg);
                if (!beforeArg.Cancel) {
                    foreach (RepPP.GroupInstance ginst in m_arrChildGroupInstances) {
                        TreeNodeGInst.FillNodes(TreeView as TreeViewCInst, Nodes, ginst, ChildNodeFactory);
                    }
                    afterArg = new TreeViewCInst.AfterChildReadEventArgs(this);
                    OnAfterChildRead(afterArg);
                }
            }
        }
    }
}
