using System;
using System.Runtime.InteropServices;
using EnvDTE;
using System.Windows.Forms;
using RepPP.Toolkit;
#if VC8
using System.Collections.Generic;
using Microsoft.VisualStudio.TemplateWizard;
#endif

//**********************************************************************
//
//    Module:             Connect.cs
//
//    Author:             Maher Najah
//    Version:            1.00.00
//    Creation:           2002-01-21
//    Modification:       2002-01-21
//
//    Description:        Implementing Rep++ Wizard entry object
//
//**********************************************************************
namespace RepPP.VSNetWizard.Windows {
#if VC8
	public class Connect : IWizard {
        /// <summary>Wizard main form</summary>
        private frmMain                     m_frmMain;
        /// <summary>Wizard kind</summary>
        private WizardRunKind               m_eWizKind;
        //**********************************************************************
        //
        /// <summary>
        /// RunStarted:                             Called when the wizard is started        
        /// </summary>
        /// <param name="automationObject">         </param>
        /// <param name="replacementsDictionary">   </param>
        /// <param name="runKind">                  </param>
        /// <param name="customParams">             </param>
        //       
        //**********************************************************************        
        public void RunStarted(object                       automationObject,
                               Dictionary<string, string>   replacementsDictionary,
                               WizardRunKind                runKind,
                               object[]                     customParams) {
            string  strPrjName;
            string  strPrjDirName;
            string  strWizType;
            DTE     dte;
            Project project;
            string  strLang;

            m_eWizKind = runKind;
            if (customParams[0].ToString().IndexOf("\\CSharp\\") != -1) {
                strLang = "CS";
            } else {
                strLang = "VB";
            }
            if (runKind == WizardRunKind.AsNewItem) {
                dte             = automationObject as DTE;
                project         = ((object[])dte.ActiveSolutionProjects)[0] as Project;
                strPrjName      = project.Name;
                strPrjDirName   = System.IO.Path.GetDirectoryName(project.FullName);
                strWizType      = "ITEM";
            } else {
                strPrjName      = replacementsDictionary["$safeprojectname$"];
                strPrjDirName   = replacementsDictionary["$destinationdirectory$"];
                strWizType      = "APP";
                replacementsDictionary.Add("$repppbindirectory$", Utility.GetRepPPBinDir());
            }
            m_frmMain = new frmMain(automationObject,
                                    strPrjName,
                                    strPrjDirName,
                                    strLang,
                                    strWizType);
            m_frmMain.ShowDialog();
            if (m_frmMain.DialogResult == System.Windows.Forms.DialogResult.Cancel) {
                m_frmMain.Dispose();
                throw new WizardCancelledException("the operation is cancelled");
            }
        }
        //**********************************************************************
        //
        /// <summary>
        /// BeforeOpeningFile:          Called before opening any item that has the 
        ///                             OpenInEditor attribute.
        /// </summary>
        /// <param name="projectItem">  Project item</param>
        //       
        //**********************************************************************
        public void BeforeOpeningFile(ProjectItem projectItem) {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectFinishedGenerating:      Called after finishing generating the ptoject
        /// </summary>
        /// <param name="project">          Project</param>
        //       
        //**********************************************************************
        public void ProjectFinishedGenerating(Project project) {
            try {
                Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;
                if (m_frmMain.DialogResult == DialogResult.OK) {
                    m_frmMain.Generate();
                }
            } finally {
                if (m_frmMain != null) {
                    m_frmMain.Dispose();
                    m_frmMain = null;
                    Cursor.Current = System.Windows.Forms.Cursors.Default;
                }
            }
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectItemFinishedGenerating:      This method is only called for item templates,
        ///                                     not for project templates.
        /// </summary>
        /// <param name="projectItem">          Project item</param>
        //       
        //**********************************************************************        
        public void ProjectItemFinishedGenerating(ProjectItem projectItem) {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectItemFinishedGenerating:      Called after the project is created
        /// </summary>
        /// <param name="projectItem">          Project item</param>
        //       
        //**********************************************************************        
        public void RunFinished() {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ShouldAddProjectItem:   This method is only called for item templates,
        //                          not for project templates.
        /// </summary>
        /// <param name="filePath"> File full name</param>
        //       
        //**********************************************************************        
        public bool ShouldAddProjectItem(string filePath) {
            bool    bRetVal = true;

            if (m_eWizKind == WizardRunKind.AsNewItem) {
                try {
                    Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;
                    if (m_frmMain.DialogResult == DialogResult.OK) {
                        m_frmMain.Generate();
                    }
                } finally {
                    if (m_frmMain != null) {
                        m_frmMain.Dispose();
                        m_frmMain = null;
                    }
                    bRetVal = false;
                    Cursor.Current = System.Windows.Forms.Cursors.Default;
                }
            }
            return(bRetVal);
        }        
	}
#else
	/// <summary>
	/// Summary description for WinConnect.
	/// </summary>
    [GuidAttribute(Global.WIZARD_GUID), ProgIdAttribute(Global.WIZARD_PRGID)] 
    public class Connect : EnvDTE.IDTWizard	{
        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public Connect() {
		}
        //**********************************************************************
        //
        /// <summary>
        /// Execute:        Occurs when a wizard is launched from either the Add Item or the New 
        ///                 Project dialog box
        /// </summary>
        /// <param name="Application">  Pointer to DTE</param>
        /// <param name="hwndOwner">    The hWnd handle for the parent of the wizard's window</param>
        /// <param name="ContextParams">An array of context parameters</param>
        /// <param name="CustomParams"> An array  of user defined parameters</param>
        /// <param name="retval">       The results of the wizard</param>
        //       
        //**********************************************************************
        public void Execute(object              Application, 
                            int                 hwndOwner, 
                            ref object[]        ContextParams, 
                            ref object[]        CustomParams, 
                            ref wizardResult    retval) {
	        frmMain oForm;
	        string  strLang = "";
	        string  strType = "";

	        if (CustomParams[0].ToString().IndexOf("LANGUAGE = VB") != -1) {
		        strLang = "VB";
	        } else if (CustomParams[0].ToString().IndexOf("LANGUAGE = CS") != -1) {
		        strLang = "CS";
	        }
	        if (CustomParams[1].ToString().IndexOf("TYPE = APPLICATION") != -1) {
		        strType = "APP";
	        } else if(CustomParams[1].ToString().IndexOf("TYPE = ITEM")  != -1) {
		        strType = "ITEM";
	        }
	        if (strType == "APP") {
		        oForm = new frmMain(Application, ContextParams[1].ToString(), ContextParams[2].ToString(), strLang, strType);
	        } else {
		        oForm = new frmMain(Application, ContextParams[1].ToString(), ContextParams[3].ToString(), strLang, strType);
	        }
	        if (oForm != null) {
		        oForm.ShowDialog();
		        oForm.Dispose();
	        } else {
		        MessageBox.Show("Enable to start Rep++ Wizard");
	        }
        }
	}
#endif
}
