//*********************************************************
//
// frmStep4:    Step 3 form: Select the HTML template file
//              HTML file.
//
// Author:		Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.Win32;
using System.IO;
using System.Xml;

namespace RepPP.VSNetWizard.ASPNet
{
	/// <summary>
	/// ASP.Net step 4 form: Select the Template file
	/// </summary>
	internal class frmStep4 : frmStepBase {
        /// <summary>Hash table of templates info. The key is the template title</summary>
        Hashtable                                   m_hashTemplateInfo;

        private System.Windows.Forms.Button         butTempDir;
        private System.Windows.Forms.ImageList      imageList1;
        private System.Windows.Forms.GroupBox       grpPreview;
        private System.Windows.Forms.PictureBox     picPreview;
        private System.Windows.Forms.TextBox        txtTemplDir;
        private System.Windows.Forms.Label          lblTemplDir;
        private System.Windows.Forms.Label          lblTemplName;
        private System.Windows.Forms.ComboBox       cboTemplName;
        private System.ComponentModel.IContainer    components;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public frmStep4() {
			InitializeComponent();
            m_hashTemplateInfo = new Hashtable();
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="frmMain">      Main form</param>
        //
        //******************************************************************
		public frmStep4(frmMain frmMain) : base(frmMain) {
			InitializeComponent();
            m_hashTemplateInfo                  = new Hashtable();
            LoadTempFolder();
            butTempDir.Click                   += new EventHandler(OnButTemplatesDirClick);
            txtTemplDir.TextChanged            += new EventHandler(OnTemplatesDirTextChanged);
            cboTemplName.SelectedIndexChanged  += new EventHandler(OnTemplateSelectedIndexChanged);
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadTempFolderv:            Load the last template folder used 
        /// </summary>
        //
        //******************************************************************
        public void LoadTempFolder() {
            string strRepPPDir;

            txtTemplDir.Text = Utility.GetRegistryKeyValue(Global.ASPNET_WIZARD_REGKEY, "ASP.NET Templates");
            if (txtTemplDir.Text == String.Empty) {
                strRepPPDir = Utility.GetRepPPDir();
                txtTemplDir.Text = Path.Combine(strRepPPDir, "HTMLDir\\Templates");
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveTempFolder:            Save the last template folder used 
        /// </summary>
        //
        //******************************************************************
        public void SaveTempFolder() {
            Utility.SetRegistryKeyValue(Global.ASPNET_WIZARD_REGKEY, "ASP.NET Templates", txtTemplDir.Text);
        }

        //******************************************************************
        //
        /// <summary>
        /// XmlTemplateFile:             Xml template file
        /// </summary>
        //
        //******************************************************************
        public string XmlTemplateFile {
            get {
                return(cboTemplName.Text != String.Empty ? 
                      ((TemplateInfo)m_hashTemplateInfo[cboTemplName.Text]).FileFullName :
                      String.Empty);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InitV:                    Initialize the form
        /// </summary>
        //
        //******************************************************************
        public override void InitV() {
            base.InitV();
            FillTemplateComboBox();
        }

        //******************************************************************
        //
        /// <summary>
        /// ValidateB:                    Validate the step
        /// </summary>
        /// <returns>
        /// True:   Valid
        /// False:  Not Valid
        /// </returns>
        //
        //******************************************************************
        public override bool ValidateB() {
            return(cboTemplName.Text.Trim() != String.Empty);
        }

        //******************************************************************
        //
        /// <summary>
        /// FillTemplateComboBox:      Fill the template  combobox
        /// </summary>
        //
        //******************************************************************
        private void FillTemplateComboBox() {
            string          strDirName;
            string[]        files;
            XmlDocument     xmlDoc;
            string          strTitle;
            string          strLang;
            string          strPreviewImage;
            string          strType;
            XmlNode         nodeTemplate;
            XmlNode         nodeHeader;
            bool            bIsGoodType;
            bool            bIsGoodLang;

            cboTemplName.Items.Clear();
            m_hashTemplateInfo.Clear();
            strDirName = txtTemplDir.Text;
            if (System.IO.Directory.Exists(strDirName)) {
                files  = System.IO.Directory.GetFiles(strDirName, "*.tmpl");
                xmlDoc = new XmlDocument();
                foreach (string strFile in files) {
                    xmlDoc.Load(strFile);
                    nodeTemplate = Utility.FindXmlNode(xmlDoc, null, "Template");
                    nodeHeader   = Utility.FindXmlNode(xmlDoc, nodeTemplate, "Header");
                    strLang      = Utility.GetXmlNodeAttributeValue(nodeHeader, "language", "c#");
                    strType      = Utility.GetXmlNodeAttributeValue(nodeHeader, "type", "application");
                    switch ((m_frmMain as frmMain).GenerationMode) {
                    case GenerationModeE.sdApplication:
                    case GenerationModeE.sdItemCard:
                        bIsGoodType = (strType.ToLower() == "application");
                        break;
                    case GenerationModeE.sdItemGroup:
                        bIsGoodType = (strType.ToLower() == "form");
                        break;
                    default:
                        bIsGoodType = false;
                        break;
                    }
                    switch (m_frmMain.ProgramLanguage) {
                    case ProgramLanguageE.sdCSharp:
                        bIsGoodLang = (strLang.ToLower() == "c#");
                        break;
                    case ProgramLanguageE.sdVisualBasic:
                        bIsGoodLang = (strLang.ToLower() == "vb");
                        break;
                    default:
                        bIsGoodLang = false;
                        break;
                    }
                    if (bIsGoodLang && bIsGoodType) {
                        strTitle        = Utility.GetXmlNodeAttributeValue(nodeHeader, "title", "");
                        strPreviewImage = Utility.GetXmlNodeAttributeValue(nodeHeader, "previewimage", "");
                        if (strPreviewImage != String.Empty) {
                            strPreviewImage = Utility.GetFullPath(strDirName, strPreviewImage);
                        }
                        if (m_hashTemplateInfo[strTitle] == null) {
                            m_hashTemplateInfo.Add(strTitle, new TemplateInfo(strFile, strTitle, strPreviewImage));
                            cboTemplName.Items.Add(strTitle);
                        }
                    }
                }
            }
            if (cboTemplName.Items.Count > 0) {
                cboTemplName.SelectedIndex = 0;
            } else {
                cboTemplName.SelectedIndex = -1;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetPreviewPicture:      Set the preview image
        /// </summary>
        //  
        //*********************************************************     
        private void SetPreviewPicture() {
            string      strImageFileName;

            picPreview.Image = null;
            strImageFileName = ((TemplateInfo)m_hashTemplateInfo[cboTemplName.Text]).PreviewImageFullName;
            if (strImageFileName != String.Empty && File.Exists(strImageFileName)) {
                picPreview.Image = Image.FromFile(strImageFileName);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButTemplatesDirClick:     Handle meta-template folder button click event
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event Arguments </param>
        //  
        //*********************************************************     
        private void OnButTemplatesDirClick(object sender, System.EventArgs e) {
            FolderBrowserDialog dlg;

            dlg              = new FolderBrowserDialog();
            dlg.SelectedPath = txtTemplDir.Text;
            if (dlg.ShowDialog() == DialogResult.OK) {
                txtTemplDir.Text = dlg.SelectedPath;
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnTemplatesDirTextChanged:          Handle templates folder text changed event
        /// </summary>
        /// <param name="sender">               Sender object   </param>
        /// <param name="e">                    Event Arguments </param>
        //  
        //*********************************************************     
        private void OnTemplatesDirTextChanged(object sender, System.EventArgs e)
        {
            FillTemplateComboBox();
            m_frmMain.NextButton.Enabled = ValidateB();
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnTempFileSelectedIndexChanged:    Handle template file selected index changed event
        /// </summary>
        /// <param name="sender">              Sender object   </param>
        /// <param name="e">                   Event Arguments </param>
        //  
        //*********************************************************     
        private void OnTemplateSelectedIndexChanged(object sender, System.EventArgs e) {
            SetPreviewPicture();
            m_frmMain.NextButton.Enabled = ValidateB();
        }

        //******************************************************************
        //
        /// <summary>
        /// TemplateInfo:       Template information class
        /// </summary>
        //
        //******************************************************************
        private class TemplateInfo {
            /// <summary>Template xml file full name</summary>
            private string              m_strFileFullName;
            /// <summary>Template title</summary>
            private string              m_strTitle;
            /// <summary>Template preview image full path</summary>
            private string              m_strPreviewImage;

            //******************************************************************
            //
            /// <summary>
            /// ctor:                   Constructor
            /// </summary>
            //
            //******************************************************************
            public TemplateInfo(string  strFileFullName,
                                string  strTitle,
                                string  strPreviewImage) {
                m_strFileFullName   = strFileFullName;
                m_strTitle          = strTitle;
                m_strPreviewImage   = strPreviewImage;
            }

            //******************************************************************
            //
            /// <summary>
            /// FileFullName:       Template xml file full name
            /// </summary>
            //
            //******************************************************************
            public string FileFullName {
                get {
                    return(m_strFileFullName);
                }
                set {
                    m_strFileFullName = value;
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// Title:              Template title
            /// </summary>
            //
            //******************************************************************
            public string Title {
                get {
                    return(m_strTitle);
                }
                set {
                    m_strTitle = value;
                }
            }

            //******************************************************************
            //
            /// <summary>
            /// PreviewImageFullName:       Template image preview full name
            /// </summary>
            //
            //******************************************************************
            public string PreviewImageFullName {
                get {
                    return(m_strPreviewImage);
                }
                set {
                    m_strPreviewImage = value;
                }
            }
        } // Class TemplateInfo
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(frmStep4));
            this.butTempDir = new System.Windows.Forms.Button();
            this.imageList1 = new System.Windows.Forms.ImageList(this.components);
            this.txtTemplDir = new System.Windows.Forms.TextBox();
            this.lblTemplDir = new System.Windows.Forms.Label();
            this.lblTemplName = new System.Windows.Forms.Label();
            this.cboTemplName = new System.Windows.Forms.ComboBox();
            this.grpPreview = new System.Windows.Forms.GroupBox();
            this.picPreview = new System.Windows.Forms.PictureBox();
            this.pnlStep.SuspendLayout();
            this.grpPreview.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlStep
            // 
            this.pnlStep.Controls.Add(this.grpPreview);
            this.pnlStep.Controls.Add(this.txtTemplDir);
            this.pnlStep.Controls.Add(this.cboTemplName);
            this.pnlStep.Controls.Add(this.lblTemplName);
            this.pnlStep.Controls.Add(this.butTempDir);
            this.pnlStep.Controls.Add(this.lblTemplDir);
            this.pnlStep.Name = "pnlStep";
            this.pnlStep.Size = new System.Drawing.Size(416, 280);
            // 
            // butTempDir
            // 
            this.butTempDir.BackColor = System.Drawing.SystemColors.Control;
            this.butTempDir.ImageIndex = 0;
            this.butTempDir.ImageList = this.imageList1;
            this.butTempDir.Location = new System.Drawing.Point(376, 16);
            this.butTempDir.Name = "butTempDir";
            this.butTempDir.Size = new System.Drawing.Size(28, 22);
            this.butTempDir.TabIndex = 4;
            this.butTempDir.Text = ". . .";
            // 
            // imageList1
            // 
            this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
            this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
            this.imageList1.TransparentColor = System.Drawing.Color.White;
            // 
            // txtTemplDir
            // 
            this.txtTemplDir.Location = new System.Drawing.Point(104, 16);
            this.txtTemplDir.Name = "txtTemplDir";
            this.txtTemplDir.Size = new System.Drawing.Size(264, 20);
            this.txtTemplDir.TabIndex = 3;
            this.txtTemplDir.Text = "";
            // 
            // lblTemplDir
            // 
            this.lblTemplDir.Location = new System.Drawing.Point(8, 16);
            this.lblTemplDir.Name = "lblTemplDir";
            this.lblTemplDir.Size = new System.Drawing.Size(96, 24);
            this.lblTemplDir.TabIndex = 3;
            this.lblTemplDir.Text = "Templates Folder";
            // 
            // lblTemplName
            // 
            this.lblTemplName.Location = new System.Drawing.Point(8, 48);
            this.lblTemplName.Name = "lblTemplName";
            this.lblTemplName.Size = new System.Drawing.Size(88, 16);
            this.lblTemplName.TabIndex = 6;
            this.lblTemplName.Text = "Template Name";
            // 
            // cboTemplName
            // 
            this.cboTemplName.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboTemplName.Location = new System.Drawing.Point(104, 48);
            this.cboTemplName.Name = "cboTemplName";
            this.cboTemplName.Size = new System.Drawing.Size(264, 21);
            this.cboTemplName.TabIndex = 4;
            // 
            // grpPreview
            // 
            this.grpPreview.Controls.Add(this.picPreview);
            this.grpPreview.Location = new System.Drawing.Point(104, 80);
            this.grpPreview.Name = "grpPreview";
            this.grpPreview.Size = new System.Drawing.Size(264, 192);
            this.grpPreview.TabIndex = 9;
            this.grpPreview.TabStop = false;
            this.grpPreview.Text = "Preview";
            // 
            // picPreview
            // 
            this.picPreview.Dock = System.Windows.Forms.DockStyle.Fill;
            this.picPreview.Location = new System.Drawing.Point(3, 16);
            this.picPreview.Name = "picPreview";
            this.picPreview.Size = new System.Drawing.Size(258, 173);
            this.picPreview.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
            this.picPreview.TabIndex = 8;
            this.picPreview.TabStop = false;
            // 
            // frmStep4
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(440, 302);
            this.Name = "frmStep4";
            this.Text = "frmWebStep1";
            this.pnlStep.ResumeLayout(false);
            this.grpPreview.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion
	} // Class frmStep4
} // Namespace
